require 'sinatra/base'
require 'sinatra/config_file'
require 'rest-client'
require 'json'

class App < Sinatra::Base
  register Sinatra::ConfigFile

  enable :sessions
  config_file 'settings/config.yml'

  # Render the first page where the user is presented with the article teaser.
  # We strongly recommend separating the article teaser and the purchase flow into separate pages.
  get '/' do
    redirect auth_url unless session[:access_token_data]
    erb :index, :locals => { :product_code => settings.product_code, :has_article_access => has_article_access?, :layout => :layout }
  end

  # Here we need to use the authorization_code that we get to exchange it for an access token to be used to carry out operations on behalf of the user
  get '/landing' do
    code = params[:code]
    raise ArgumentError.new('autorization code required') unless code
    access_token_data = get_access_token_data(code)

    # we store the access token data in our local session and then we send the user back to the index page to carry on browsing
    session[:access_token_data] = access_token_data if access_token_data
    redirect '/'
  end

  # Here we establish a payment session towards payway and klarna.
  # This is also where the purchase flow will be presented to the user.
  get '/buy' do
    payload = {
        :code => settings.product_code,
        :period_type => settings.period_type,
        :confirmation_url => settings.confirmation_uri
    }
    begin
      response = create_payment_session(payload)
      session[:klarna_payments_session_id] = response.klarna_payments_session_id
      erb :buy, :locals => {:klarna_client_token => response.klarna_client_token }, :layout => :layout
    rescue StandardError
      erb :error, :layout => :layout
    end
  end

  # The user approves the purchase. We use the authorization token you receive from klarna to complete the purchase in payway and klarna
  post '/buy' do
    payload = {
        :klarna_authorization_token => params[:klarna_authorization_token],
        :klarna_payments_session_id => session[:klarna_payments_session_id],
        :browser_ip => request.ip,
        :browser_language => request.env['HTTP_ACCEPT_LANGUAGE'].split(',')[0],
        :browser_user_agent => request.user_agent
    }
    begin
      response = place_order(payload)
      # Here we must send the user's browser to klarna, klarna will in turn take care of sending the user to the confirmation url given in the request above.
      redirect response.redirect_url
    rescue StandardError
      erb :error, :layout => :layout
    end
  end

  get '/confirmation' do
    begin
      erb :confirmation, :layout => :layout
    rescue StandardError
      erb :error, :layout => :layout
    end
  end

  private

  def create_payment_session(payload)
    headers = {
        'Authorization' => "Bearer #{session[:access_token_data][:access_token]}",
        'Content-Type' => 'application/json',
        'Accept' => 'application/json'
    }
    response = RestClient.post payment_session_url, payload.to_json, headers
    json = JSON.parse(response.body, :symbolize_names => true)
    OpenStruct.new({:klarna_payments_session_id => json[:item][:klarna_payments_session_id], :klarna_client_token => json[:item][:klarna_client_token]})
  rescue RestClient::ExceptionWithResponse => e
    parse_error(e.code, JSON.parse(e.response.body, :symbolize_names => true))
    raise e
  end

  def place_order(payload)
    headers = {
        'Authorization' => "Bearer #{session[:access_token_data][:access_token]}",
        'Content-Type' => 'application/json',
        'Accept' => 'application/json'
    }
    response = RestClient.post place_order_url, payload.to_json, headers
    json = JSON.parse(response.body, :symbolize_names => true)
    OpenStruct.new({:redirect_url => json[:item][:redirect_url]})

  rescue RestClient::ExceptionWithResponse => e
    parse_error(e.code, JSON.parse(e.response.body, :symbolize_names => true))
    raise e
  end

  def has_article_access?
    headers = {
        'Authorization' => "Bearer #{session[:access_token_data][:access_token]}",
        'Accept' => 'application/json'
    }
    response = RestClient.get active_product_codes_url, headers

    json = JSON.parse(response.body, :symbolize_names => true)
    json[:item][:active_products].include?(settings.product_code)

  rescue RestClient::ExceptionWithResponse => e
    parse_error(e.code, JSON.parse(e.response.body, :symbolize_names => true))
    raise e
  end

  def get_access_token_data(code)
    params = {
        :grant_type => "authorization_code",
        :code => code,
        :client_id => settings.client_id,
        :client_secret => settings.client_secret,
        :redirect_uri => settings.redirect_uri,
        :scope => settings.scopes
    }
    response = RestClient.post access_token_url, params
    JSON.parse(response.body, :symbolize_names => true)
  rescue RestClient::ExceptionWithResponse => e
    parse_error(e.code, JSON.parse(e.response.body, :symbolize_names => true))
    raise e
  end

  def auth_url
    "#{settings.auth_uri}?client_id=#{settings.client_id}&oid=#{settings.organisation_id}&response_type=code&redirect_uri=#{settings.redirect_uri}&scope=#{settings.scopes}"
  end

  def access_token_url
    "#{settings.access_token_uri}"
  end

  def payment_session_url
    "#{settings.payment_session_uri}"
  end

  def place_order_url
    "#{settings.place_order_uri}"
  end

  def active_product_codes_url
    "#{settings.active_product_codes_uri}"
  end

  private

  def parse_error(code, error)
    "#{code} - #{error}"
  end
end